#include <dpf-internal.h>
#include <string.h>

#define IDENTITY_MASK8	0xffUL
#define IDENTITY_MASK16 0xffffUL
#define IDENTITY_MASK32	0xffffffffUL

void dpf_init(struct dpf_ir *ir) {
	memset(ir, 0, sizeof *ir);
}

static inline struct ir *alloc(struct dpf_ir *ir) {
	if(ir->irn >= DPF_MAXELEM)
		fatal(Code size exceeded.  Increase DPF_MAX.);
	return &ir->ir[ir->irn++];
}

static inline void 
mkeq(struct dpf_ir *ir, uint8 nbits, uint16 offset, uint32 mask, uint32 val) {
	struct eq *eq;

	eq = &alloc(ir)->u.eq;
	eq->op = DPF_OP_EQ;
	/* add in relative offset now so don't have to do at demux time. */
	eq->offset = offset + ir->moffset;
	eq->val = val;
	eq->mask = mask;
	eq->nbits = nbits;
}

void dpf_eq8(struct dpf_ir *ir, uint16 offset, uint8 val) 
	{ dpf_meq8(ir, offset, IDENTITY_MASK8, val); }
void dpf_eq16(struct dpf_ir *ir, uint16 offset, uint16 val) 
	{ dpf_meq16(ir, offset, IDENTITY_MASK16, val); }
void dpf_eq32(struct dpf_ir *ir, uint16 offset, uint32 val) 
	{ dpf_meq32(ir, offset, IDENTITY_MASK32, val); }

/* Is the message quantity of nbits at offset equal to val? */
void dpf_meq8(struct dpf_ir *ir, uint16 offset, uint8 mask, uint8 val) {
	if((mask & IDENTITY_MASK8) != mask)
		fatal(Mask exceeds width of type.);
	mkeq(ir, 8, offset, mask, val);
}

void dpf_meq16(struct dpf_ir *ir, uint16 offset, uint16 mask, uint16 val) {
	if((mask & IDENTITY_MASK16) != mask)
		fatal(Mask exceeds width of type.);
	mkeq(ir, 16, offset, mask, val);
}

void dpf_meq32(struct dpf_ir *ir, uint16 offset, uint32 mask, uint32 val) {
	if((mask & IDENTITY_MASK32) != mask)
		fatal(Mask exceeds width of type.);
	mkeq(ir, 32, offset, mask, val);
}

static inline void
mkshift(struct dpf_ir *ir, uint8 nbits, uint16 offset, uint32 mask, uint8 shift) {
	struct shift *s;

	s = &alloc(ir)->u.shift;
	s->op = DPF_OP_SHIFT;
	s->shift = shift;
	s->offset = offset;
	s->mask = mask;
	s->nbits = nbits;
	ir->moffset = 0;	/* relative offset is zero. */
	
}

/* shift the base message pointer by nbits */
void dpf_shift8(struct dpf_ir *ir, uint16 offset, uint8 shift) 
	{ dpf_mshift8(ir, offset, IDENTITY_MASK8, shift); }
void dpf_shift16(struct dpf_ir *ir, uint16 offset, uint8 shift) 
	{ dpf_mshift16(ir, offset, IDENTITY_MASK16, shift); }
void dpf_shift32(struct dpf_ir *ir, uint16 offset, uint8 shift) 
	{ dpf_mshift32(ir, offset, IDENTITY_MASK32, shift); }

void dpf_mshift8(struct dpf_ir *ir, uint16 offset, uint8 mask, uint8 shift) 
	{ mkshift(ir, 8, offset, mask, shift); }
void dpf_mshift16(struct dpf_ir *ir, uint16 offset, uint16 mask, uint8 shift) 
	{ mkshift(ir, 16, offset, mask, shift); }
void dpf_mshift32(struct dpf_ir *ir, uint16 offset, uint32 mask, uint8 shift) 
	{ mkshift(ir, 32, offset, mask, shift); }

void dpf_shifti(struct dpf_ir *ir, uint16 offset) 
	{ ir->moffset += offset; }
